/*
  cubexcgiout.cpp
  Cubex by Eric Dietz (c) 2003
  Cube Solver, HTML output.
  Notes: readme.txt  Email: root@wrongway.org
  NOTE: This program is unaffiliated with the Rubik's Cube Trademark.
*/

#include <cstdio>
#include <string>
using namespace std;
#include "cubex.h"

// shared variables...
Cubex thecube;
Cubex thetocube;
string thesolution = "";
int themov[10];
string col[7];
string htm = "";
string exc = "";
string inp = "";
bool disp_diagrams;
bool disp_graphics;
bool solveto;
string cmd = "";
string adv = "";
int cmdc;
// show a cube configuration in html based on Cub[] model.
void RenderHtml(Cubex somecube) {
  printf("<table border=0 cellpadding=0 cellspacing=1><tr><td></td><td>\n");
  printf("<table border=1 width=32 height=32 bordercolor=#000000 cellpadding=0 cellspacing=0>\n");
  for (int i = -1; i <= 1; i++) {
    printf("<tr>");
    for (int j = -1; j <= 1; j++) {
      printf("<td bgcolor=%s><table><tr><td></td></tr></table></td>", col[*somecube.face(j,2,-i)].c_str());
    }
    printf("</tr>\n");
  }
  printf("</table></td><td></td><td></td></tr><tr><td>\n");
  printf("<table border=1 width=32 height=32 bordercolor=#000000 cellpadding=0 cellspacing=0>\n");
  for (int i = -1; i <= 1; i++) {
    printf("<tr>");
    for (int j = -1; j <= 1; j++) {
      printf("<td bgcolor=%s><table><tr><td></td></tr></table></td>", col[*somecube.face(-2,-i,-j)].c_str());
    }
    printf("</tr>\n");
  }
  printf("</table></td><td>\n");
  printf("<table border=1 width=32 height=32 bordercolor=#000000 cellpadding=0 cellspacing=0>\n");
  for (int i = -1; i <= 1; i++) {
    printf("<tr>");
    for (int j = -1; j <= 1; j++) {
      printf("<td bgcolor=%s><table><tr><td></td></tr></table></td>", col[*somecube.face(j,-i,-2)].c_str());
    }
    printf("</tr>\n");
  }
  printf("</table></td><td>\n");
  printf("<table border=1 width=32 height=32 bordercolor=#000000 cellpadding=0 cellspacing=0>\n");
  for (int i = -1; i <= 1; i++) {
    printf("<tr>");
    for (int j = -1; j <= 1; j++) {
      printf("<td bgcolor=%s><table><tr><td></td></tr></table></td>", col[*somecube.face(2,-i,j)].c_str());
    }
    printf("</tr>\n");
  }
  printf("</table></td><td>\n");
  printf("<table border=1 width=32 height=32 bordercolor=#000000 cellpadding=0 cellspacing=0>\n");
  for (int i = -1; i <= 1; i++) {
    printf("<tr>");
    for (int j = -1; j <= 1; j++) {
      printf("<td bgcolor=%s><table><tr><td></td></tr></table></td>", col[*somecube.face(-j,-i,2)].c_str());
    }
    printf("</tr>\n");
  }
  printf("</table></td></tr><tr><td></td><td>\n");
  printf("<table border=1 width=32 height=32 bordercolor=#000000 cellpadding=0 cellspacing=0>\n");
  for (int i = -1; i <= 1; i++) {
    printf("<tr>");
    for (int j = -1; j <= 1; j++) {
      printf("<td bgcolor=%s><table><tr><td></td></tr></table></td>", col[*somecube.face(j,-2,i)].c_str());
    }
    printf("</tr>\n");
  }
  printf("</table></td><td></td><td></td></tr></table>\n");
}
// make the body of the html document showing the solution
void ShowHtml() {
  if (!thecube.cubeinit) return;
  int m = 0, h, l, maxcol;
  string a, img = "";
  //maxcol = (disp_diagrams || !disp_graphics) ? 3 : 5;
  maxcol = 3;
  if (disp_diagrams) {
    printf("<script language=JavaScript>\n");
    printf("// non-script <!--\n");
    printf("var mov = new Array( ");
    for (int i = 0; i <= 9; i++) {
      printf("%i ", themov[i]);
      if (i < 9) {
        printf(", ");
      }
    }
    printf(");\n");
    printf(
"\
// Cubex JavaScript\n\
function ElementGet(xx) {\n\
  return document.getElementById(xx);\n\
}\n\
function ShowStep(name) {\n\
  var step = ElementGet(name);\n\
  if (step.style.display == '')\n\
    step.style.display = 'none';\n\
  else\n\
    step.style.display = '';\n\
}\n\
function ShowGroup(group, see) {\n\
  var step, vis, first = 1, last, i;\n\
  if (see == 'true')\n\
    vis = '';\n\
  else\n\
    vis = 'none';\n\
  for (i = 1; i <= group - 1; i++)\n\
    first = first + mov[i];\n\
  last = first + mov[group] - 1;\n\
  for (i = first; i <= last; i++) {\n\
    step = ElementGet('step' + i);\n\
    step.style.display = vis;\n\
  }\n\
}\n\
function ShowAll(see) {\n\
  var step, vis, i;\n\
  if (see == 'true')\n\
    vis = '';\n\
  else\n\
    vis = 'none';\n\
  for (i = 1; i <= mov[0]; i++) {\n\
    step = ElementGet('step' + i);\n\
    step.style.display = vis;\n\
  }\n\
}\n\
"
    );
    printf("// -->\n");
    printf("</script>\n");
  }
  printf("<noscript>\n");
  printf("<b>Your browser has JavaScript disabled.</b><br>\n");
  printf("</noscript>\n");
  printf("<table border=0><tr><td>\n");
  printf("Starting Position:<br>\n");
  RenderHtml(thecube);
  if (solveto) {
    printf("</td><td>\n");
    printf("Ending Position:<br>\n");
    RenderHtml(thetocube);
  }
  printf("</td></tr></table>\n");
  printf("<br>\n");
  printf("<a href=%ssyntax.html target=_blank>View Syntax Notes</a>,\n", htm.c_str());
  printf("<a href=%sworked.html target=_blank>About the Solver</a>,\n", htm.c_str());
  printf("<a href=%s?!%s%s%s%s target=_blank>Input Screen</a><br>\n", inp.c_str(), cmd.c_str(), thecube.cenfix?"!fixcen":"", solveto?"!solveto":"", adv.c_str());
  printf("<br>\n");
  if (disp_diagrams) {
    printf("<a href=javascript:ShowAll('true')>Expand All</a>,\n");
    printf("<a href=javascript:ShowAll('false')>Collapse All</a><br>\n");
  } else {
    printf("<a href=%s?%s!withdiagrams><b>Click here to see a diagrammed version!</b></a>\n", exc.c_str(), cmd.c_str());
  }
  if (!disp_graphics) {
    printf("<a href=%s?%s!withgraphics><b>Click here for graphical version!</b></a>\n", exc.c_str(), cmd.c_str());
  }
  if (!disp_diagrams || !disp_graphics)
    printf("<br>\n");
  printf("<b>Complete solution: %i moves</b><br>\n", thecube.mov[0]);
  printf("Layered Break-down -<br>\n");
  printf("<br>\n");
  for (int i = 1; i <= 9; i++) {
    if (i == 8 && !thecube.cenfix) continue;
    if (i == 9 && !solveto) continue;
    printf("<b>");
    switch (i) {
      case 1: printf("First-Layer Edges"); break;
      case 2: printf("First-Layer Corners"); break;
      case 3: printf("Second-Layer (Edges)"); break;
      case 4: printf("Third-Layer Edges Orient"); break;
      case 5: printf("Third-Layer Edges Position"); break;
      case 6: printf("Third-Layer Corners Position"); break;
      case 7: printf("Third-Layer Corners Orient"); break;
      case 8: printf("Center Rotation Fix"); break;
      case 9: printf("Cube Rotations"); break;
    }
    printf(": %i moves</b>", themov[i]);
    if (disp_diagrams) {
      printf("\n");
      printf("<small><a href=javascript:ShowGroup(%i,'true')>Expand Group</a>,\n", i);
      printf("<a href=javascript:ShowGroup(%i,'false')>Collapse Group</a></small><br>\n", i);
    } else {
      printf("<br>\n");
    }
    printf("<table><tr>\n");
    h = 0;
    l = 0;
    for (int j = 1; j <= themov[i]; j++) {
      h++; m++;
      if (h > maxcol) {
        h = 1;
        l = !l;
        if (l) {
          printf("</tr><tr bgcolor=#D0D0D0>");
        }
        else {
          printf("</tr><tr>");
        }
      }
      printf("<td valign=top nowrap>\n");
      a = thesolution.substr(m * 3 - 3, 3);
      if      (a == "UL.") { thecube.UL(); img = "ul"; a = "Top Left"; }
      else if (a == "UR.") { thecube.UR(); img = "ur"; a = "Top Right"; }
      else if (a == "DL.") { thecube.DL(); img = "dl"; a = "Bottom Left"; }
      else if (a == "DR.") { thecube.DR(); img = "dr"; a = "Bottom Right"; }
      else if (a == "LU.") { thecube.LU(); img = "lu"; a = "Left Up"; }
      else if (a == "LD.") { thecube.LD(); img = "ld"; a = "Left Down"; }
      else if (a == "RU.") { thecube.RU(); img = "ru"; a = "Right Up"; }
      else if (a == "RD.") { thecube.RD(); img = "rd"; a = "Right Down"; }
      else if (a == "FC.") { thecube.FC(); img = "fc"; a = "Front Clockwise"; }
      else if (a == "FA.") { thecube.FA(); img = "fa"; a = "Front Counterclockwise"; }
      else if (a == "BC.") { thecube.BC(); img = "bc"; a = "Rear Clockwise"; }
      else if (a == "BA.") { thecube.BA(); img = "ba"; a = "Rear Counterclockwise"; }
      else if (a == "CL.") { thecube.CL(); img = "cl"; a = "Whole Cube Left"; }
      else if (a == "CR.") { thecube.CR(); img = "cr"; a = "Whole Cube Right"; }
      else if (a == "CU.") { thecube.CU(); img = "cu"; a = "Whole Cube Up"; }
      else if (a == "CD.") { thecube.CD(); img = "cd"; a = "Whole Cube Down"; }
      else if (a == "CC.") { thecube.CC(); img = "cc"; a = "Whole Cube Clockwise"; }
      else if (a == "CA.") { thecube.CA(); img = "ca"; a = "Whole Cube Counterclockwise"; }
      if      (!disp_graphics && !disp_diagrams) {
        printf("%i. %s,\n", m, a.c_str());
      }
      else if (disp_graphics && !disp_diagrams) {
        printf("%i. <img src=%s%s.gif> %s,\n", m, htm.c_str(), img.c_str(), a.c_str());
      }
      else if (!disp_graphics && disp_diagrams) {
        printf("<a href=javascript:ShowStep('step%i')>%i. %s</a>,\n", m, m, a.c_str());
        printf("<div id=step%i style=\"display: none;\">\n", m);
        RenderHtml(thecube);
        if (m == thecube.mov[0] && thecube.cenfix)
          printf("(centers now fixed)\n");
        printf("</div>\n");
      }
      else if (disp_graphics && disp_diagrams) {
        printf("<a href=javascript:ShowStep('step%i')>%i. <img src=%s%s.gif border=0> %s</a>,\n", m, m, htm.c_str(), img.c_str(), a.c_str());
        printf("<div id=step%i style=\"display: none;\">\n", m);
        RenderHtml(thecube);
        if (m == thecube.mov[0] && thecube.cenfix)
          printf("(centers now fixed)\n");
        printf("</div>\n");
      }
      printf("</td>");
    }
    if (h > 0) {
      for (int j = h + 1; j <= maxcol; j++) {
        printf("<td>\n");
        printf("</td>");
      }
    }
    printf("</tr></table><br>\n");
  }
  if (disp_diagrams) {
    printf("<a href=javascript:ShowAll('true')>Expand All</a>,\n");
    printf("<a href=javascript:ShowAll('false')>Collapse All</a><br>\n");
  } else {
    printf("<a href=%s?%s!withdiagrams><b>Click here to see a diagrammed version!</b></a>\n", exc.c_str(), cmd.c_str());
  }
  if (!disp_graphics) {
    printf("<a href=%s?%s!withgraphics><b>Click here for graphical version!</b></a>\n", exc.c_str(), cmd.c_str());
  }
  if (!disp_diagrams || !disp_graphics)
    printf("<br>\n");
  printf("<br>\n");
  printf("<a href=%ssyntax.html target=_blank>View Syntax Notes</a>,\n", htm.c_str());
  printf("<a href=%sworked.html target=_blank>About the Solver</a>,\n", htm.c_str());
  printf("<a href=%s?!%s%s%s%s target=_blank>Input Screen</a><br>\n", inp.c_str(), cmd.c_str(), thecube.cenfix?"!fixcen":"", solveto?"!solveto":"", adv.c_str());
  printf("Sorry for the various inefficiencies!<br>\n");
  printf("<a href=%sworked.html target=_blank>\n", htm.c_str());
  printf("If this helped you, please click here.<br>\n");
  printf("<img src=%sworked.gif border=0></a><br>\n", htm.c_str());
}
// take the command-line parameter and make a cube out of it
int ScrambleCubeGet() {
  string clr[7]; int cen[6]; string cm = ""; int n;
  thecube.cubeinit = false;
  if (cmdc < 2) { return 1; }
  cm = cmd;
  if (cm.substr(0, 5) != "cube-" || cm.length() < 59) {
    return 1;
  }
  cm = cm.substr(5, cm.length() - 5);
  for (int i = 1; i <= 54; i++) {
    if (cm.at(i-1)-48 < 1 || cm.at(i-1)-48 > 6) {
      return 1;
    }
  }
  for (int i = -1; i <= 1; i++) {
    for (int j = -1; j <= 1; j++) {
      *thecube.face(j, 2, -i) = cm.at(i*3+j+4)-48;
      *thecube.face(j, -i, -2) = cm.at(i*3+j+13)-48;
      *thecube.face(-2, -i, -j) = cm.at(i*3+j+22)-48;
      *thecube.face(-j, -i, 2) = cm.at(i*3+j+31)-48;
      *thecube.face(2, -i, j) = cm.at(i*3+j+40)-48;
      *thecube.face(j, -2, i) = cm.at(i*3+j+49)-48;
    }
  }
  thecube.cubeinit = true;
// to get the optional custom colors and detailed output setting:
  cm = cm.substr(54, cm.length() - 54);
  while (cm.length() > 0) {
    if (cm.substr(0, 9) == "!centers-" && cm.length() >= 15) {
      cm = cm.substr(9, cm.length() - 9);
      n = 1;
      for (int i = 0; i <= 5; i++) {
        if (cm.at(i)-48 < 0 || cm.at(i)-48 > 3)
          n = 0;
      }
      if (n == 1) {
        thecube.cenfix = true;
        adv = "!adv";
        *thecube.face(0, 1, 0) = cm.at(0)-48;
        *thecube.face(0, 0, -1) = cm.at(1)-48;
        *thecube.face(-1, 0, 0) = cm.at(2)-48;
        *thecube.face(0, 0, 1) = cm.at(3)-48;
        *thecube.face(1, 0, 0) = cm.at(4)-48;
        *thecube.face(0, -1, 0) = cm.at(5)-48;
      }
      cm = cm.substr(6, cm.length() - 6);
    } else if (cm.substr(0, 8) == "!tocube-" && cm.length() >= 62) {
      cm = cm.substr(8, cm.length() - 8);
      solveto = true;
      for (int i = 1; i <= 54 && solveto == true; i++) {
        if (cm.at(i-1)-48 < 1 || cm.at(i-1)-48 > 6) {
          solveto = false;
        }
      }
      if (solveto == true) {
        adv = "!adv";
        for (int i = -1; i <= 1; i++) {
          for (int j = -1; j <= 1; j++) {
            *thetocube.face(j, 2, -i) = cm.at(i*3+j+4)-48;
            *thetocube.face(j, -i, -2) = cm.at(i*3+j+13)-48;
            *thetocube.face(-2, -i, -j) = cm.at(i*3+j+22)-48;
            *thetocube.face(-j, -i, 2) = cm.at(i*3+j+31)-48;
            *thetocube.face(2, -i, j) = cm.at(i*3+j+40)-48;
            *thetocube.face(j, -2, i) = cm.at(i*3+j+49)-48;
          }
        }
      }
      cm = cm.substr(54, cm.length() - 54);
    } else if (cm.substr(0, 11) == "!tocenters-" && cm.length() >= 17) {
      cm = cm.substr(11, cm.length() - 11);
      n = 1;
      for (int i = 0; i <= 5; i++) {
        if (cm.at(i)-48 < 0 || cm.at(i)-48 > 3)
          n = 0;
      }
      if (n == 1) {
        thetocube.cenfix = true;
        adv = "!adv";
        *thetocube.face(0, 1, 0) = cm.at(0)-48;
        *thetocube.face(0, 0, -1) = cm.at(1)-48;
        *thetocube.face(-1, 0, 0) = cm.at(2)-48;
        *thetocube.face(0, 0, 1) = cm.at(3)-48;
        *thetocube.face(1, 0, 0) = cm.at(4)-48;
        *thetocube.face(0, -1, 0) = cm.at(5)-48;
      }
      cm = cm.substr(6, cm.length() - 6);
    } else if (cm.substr(0, 8) == "!colors-" && cm.length() >= 44) {
      cm = cm.substr(8, cm.length() - 8);
      n = 1;
      for (int i = 1; i <= 6; i++) {
        clr[i] = "#";
        for (int j = 0; j <= 5; j++) {
          if ((cm.at(j)-48 < 0 || cm.at(j)-48 > 9) &&
           (cm.at(j)-65 < 0 || cm.at(j)-65 > 6) &&
           (cm.at(j)-97 < 0 || cm.at(j)-97 > 6))
            n = 0;
          else clr[i] += cm.at(j);
        }
        cm = cm.substr(6, cm.length() - 6);
      }
      if (n == 1) {
        adv = "!adv";
        for (int i = 1; i <= 6; i++)
          col[i] = clr[i];
      }
    } else if (cm.substr(0, 13) == "!withdiagrams") {
      disp_diagrams = true;
      cm = cm.substr(13, cm.length() - 13);
    } else if (cm.substr(0, 13) == "!withgraphics") {
      disp_graphics = true;
      cm = cm.substr(13, cm.length() - 13);
    } else {
      cmd = cmd.substr(0, cmd.length() - cm.length());
      cm = "";
    }
  }
  if (solveto && (!thecube.cenfix || !thetocube.cenfix)) {
    thecube.cenfix = false;
    thetocube.cenfix = false;
  }
  return 0;
}
// read some defaults from optional .ini file, and reset cube
void GetDefaults() {
  char f[256], *ini = "cubex.ini"; string s;
  thecube.ResetCube();
  for (int i = 0; i <= 9; i++)
    themov[i] = 0;
  col[1] = "#FFFFFF"; col[2] = "#FF8000"; col[3] = "#0000FF";
  col[4] = "#FF0000"; col[5] = "#00FF00"; col[6] = "#FFFF00";
  htm = "../../cube/";
  exc = "cubexcgiout";
  inp = "cubexcgiin";
  disp_diagrams = false;
  disp_graphics = false;
  solveto = false;
  FILE *fp = fopen(ini, "r");
  if (fp != NULL) {
    while (fgets(f, sizeof(f), fp)) {
      s = f;
      if (s.substr(0, 9) == "sitepath=") {
        htm = "";
        for (int i = 9; i <= s.length() - 1; i++) {
          if (s.at(i)-32 >= 0)
            htm += s.at(i);
          else break;
        }
      }
      /*
      else if (s.substr(0, 9) == "execname=") {
        exc = "";
        for (int i = 9; i <= s.length() - 1; i++) {
          if (s.at(i)-32 >= 0)
            exc += s.at(i);
          else break;
        }
      }
      */
    }
    fclose(fp);
  }
}
// turn an error code into a message...
void ReportError(int e) {
  switch(e) {
  case 1:
    printf("cubelet error - incorrect cubelets - cube mispainted.\n");
    break;
  case 2:
    printf("parity error - nondescript - cube misassembled.\n");
    break;
  case 3:
    printf("parity error - center rotation - cube misassembled.\n");
    printf("<br><i>hint</i>: if you get this error, perhaps try turning off\n");
    printf("'modify centers' in the input section, or else just double-check them\n");
    break;
  case 4:
    printf("cubelet error - backward centers or corners - cube mispainted.\n");
    break;
  case 5:
    printf("parity error - edge flipping - cube misassembled.\n");
    break;
  case 6:
    printf("parity error - edge swapping - cube misassembled.\n");
    break;
  case 7:
    printf("parity error - corner rotation - cube misassembled.\n");
    break;
  }
}
// main function...
int main(int argc, char *argv[]) {
  cmdc = argc;
  if (cmdc >= 2) cmd = argv[1];
  GetDefaults();
  printf("Content-Type: text/html\n\n");
  printf("<html>\n");
  printf("<!-- Generated by Cubex - Cube Solver -->\n");
  printf("<head>\n");
  printf("<title>Cube Solution</title>\n");
  printf("<meta name=Generator content=Cubex_by_Eric_Dietz>\n");
  printf("</head>\n");
  printf("<body bgcolor=#FFFFFF text=#000040 link=#000080 vlink=#800080>\n");
  printf("<font face=arial>\n");
  printf("<table width=90%% border=0 align=center><tr><td>\n");
  printf("<font size=3><p align=left>\n");
  printf("Cube Solver by <a href=mailto:root@wrongway.org>Eric Dietz</a> (c) 2003<br>\n");
  printf("Version %s<br>\n", Cubex::ver);
  printf("<br>\n");
  if (ScrambleCubeGet() != 0) {
    printf("ERROR: non-protocol input entered.\n");
    printf("Try using the Cubex solve.html to input cubes,\n");
    printf("or read documentation for command-line parameter.\n");
    printf("<br>\n");
    printf("Client sent:\n");
    printf("%s\n", cmd.c_str());
    printf("<br>\n");
  }
  else {
    int x, y;
    x = thecube.SolveCube();
    if (solveto) {
      y = thetocube.SolveCube();
    } else {
      y = 0;
    }
    if (x > 0) {
      printf("Error found in starting position:\n");
      ReportError(x);
      printf("<br>\n");
    }
    if (y > 0) {
      printf("Error found in ending position:\n");
      ReportError(y);
      printf("<br>\n");
    }
    if (x == 0 && y == 0) {
      if (solveto) {
        Cubex tempcube;
        tempcube = thetocube;
        string XtoY = "";
        string YtoX = "";
        switch (tempcube.FindCent(*thecube.face(0, 2, 0))) {
        case 1:
          XtoY = "CC."; YtoX = "CA."; tempcube.CA();
          break;
        case -1:
          XtoY = "CA."; YtoX = "CC."; tempcube.CC();
          break;
        case 2:
          break;
        case -2:
          XtoY = "CA.CA."; YtoX = "CC.CC."; tempcube.CC(); tempcube.CC();
          break;
        case 3:
          XtoY = "CU."; YtoX = "CD."; tempcube.CD();
          break;
        case -3:
          XtoY = "CD."; YtoX = "CU."; tempcube.CU();
          break;
        }
        switch (tempcube.FindCent(*thecube.face(0, 0, -2))) {
        case 1:
          XtoY = "CR." + XtoY; YtoX += "CL."; tempcube.CL();
          break;
        case -1:
          XtoY = "CL." + XtoY; YtoX += "CR."; tempcube.CR();
          break;
        case -2:
          y = -1;
          break;
        case 3:
          XtoY = "CR.CR." + XtoY; YtoX += "CL.CL."; tempcube.CL(); tempcube.CL();
          break;
        case -3:
          break;
        }
        if
        ((*thecube.face(-2, 0, 0) != *tempcube.face(-2, 0, 0)) ||
         (*thecube.face(2, 0, 0) != *tempcube.face(2, 0, 0)) ||
         (*thecube.face(0, -2, 0) != *tempcube.face(0, -2, 0)) ||
         (*thecube.face(0, 2, 0) != *tempcube.face(0, 2, 0)) ||
         (*thecube.face(0, 0, -2) != *tempcube.face(0, 0, -2)) ||
         (*thecube.face(0, 0, 2) != *tempcube.face(0, 0, 2)) ) {
          y = -1;
        }
        if (y == 0) {
          int t;
          tempcube.ResetCube();
          tempcube.solution = thetocube.solution;
          tempcube.solution += YtoX;
          for (int i = thecube.mov[0]; i >= 1; i--) {
            tempcube.solution += thecube.solution.at(i*3-3);
            if      (thecube.solution.at(i*3-2) == 'L') tempcube.solution += 'R';
            else if (thecube.solution.at(i*3-2) == 'R') tempcube.solution += 'L';
            else if (thecube.solution.at(i*3-2) == 'U') tempcube.solution += 'D';
            else if (thecube.solution.at(i*3-2) == 'D') tempcube.solution += 'U';
            else if (thecube.solution.at(i*3-2) == 'C') tempcube.solution += 'A';
            else if (thecube.solution.at(i*3-2) == 'A') tempcube.solution += 'C';
            tempcube.solution += '.';
          }
          tempcube.mov[0] = thecube.mov[0] + (YtoX.length() / 3) + thetocube.mov[0];
          tempcube.cenfix = thecube.cenfix;
          tempcube.DoSolution();
          tempcube.SolveCube();
          thecube.solution = tempcube.solution + XtoY;
          thecube.mov[0] = tempcube.mov[0] + (XtoY.length() / 3);
          for (int i = 1; i <= 8; i++) {
            thecube.mov[i] = tempcube.mov[i];
          }
          tempcube = thecube;
          tempcube.DoSolution();
          if (tempcube != thetocube) {
            if (tempcube.cenfix) tempcube.cenfix = false;
            if (tempcube == thetocube) {
              printf("Warning - center fixing may be inaccurate since the "
                     "centers on the source and destination cubes are in "
                     "different positions.<br><br>\n");
            }
            else {
              y = -1;
            }
          }
          if (y == 0) {
            thesolution = thecube.solution;
            for (int i = 0; i <= 8; i++)
              themov[i] = thecube.mov[i];
            themov[9] = (XtoY.length() / 3);
            ShowHtml();
          }
        }
        else {
          printf("Error - starting and ending positions are not compatible.\n");
          printf("The positions use different color schemes and therefore\n");
          printf("the first position can not achieve the second position.\n");
          printf("<br>\n");
        }
      }
      else {
        thesolution = thecube.solution;
        for (int i = 0; i <= 8; i++)
          themov[i] = thecube.mov[i];
        ShowHtml();
      }
    }
    if (x != 0 || y != 0) {
      printf("Your Entry:<br>\n");
      printf("<table border=0><tr><td>\n");
      printf("Starting Position:<br>\n");
      RenderHtml(thecube);
      if (solveto) {
        printf("</td><td>\n");
        printf("Ending Position:<br>\n");
        RenderHtml(thetocube);
      }
      printf("</td></tr></table>\n");
      printf("<br>\n");
      printf("<a href=%s?!%s%s%s%s target=_blank>Input Screen</a><br>\n", inp.c_str(), cmd.c_str(), thecube.cenfix?"!fixcen":"", solveto?"!solveto":"", adv.c_str());
    }
  }
  printf("</td></tr></table>\n");
  printf("</body>\n");
  printf("</html>\n");
  return 0;
}
